@@ -19,13 +19,11 @@
 #  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 #  SOFTWARE.
 
-
 import os
 import time
 import binascii
 import subprocess
-
-
+import config as gol
 import cklink
 
 try:
@@ -37,6 +35,7 @@ from libs.bflb_utils import app_path
 
 dir_dll = os.path.join(app_path, "utils/cklink")
 
+
 class BflbCKLinkPort(object):
 
     def __init__(self, vid=0, pid=0):
@@ -51,14 +50,15 @@ class BflbCKLinkPort(object):
         self._chipname = "bl808"
         self.vid = vid
         self.pid = pid
-        self.link = None
+        #self.link = None
+        self.link = gol.obj_cklink
 
     def if_init(self, device, sn, rate, chiptype="bl808", chipname="bl808"):
         if self._inited is False:
             dev = device.split("|")
             vid = int(dev[0].replace("0x", ""), 16)
             pid = int(dev[1].replace("0x", ""), 16)
-            serial = str(sn) 
+            serial = str(sn)
             bflb_utils.printf("SN is " + serial)
             sub_module = __import__("libs." + chiptype, fromlist=[chiptype])
             self._cklink_shake_hand_addr = sub_module.cklink_load_cfg.cklink_shake_hand_addr
@@ -70,23 +70,33 @@ class BflbCKLinkPort(object):
             self._inited = True
             self._chiptype = chiptype
             self._chipname = chipname
-            self.link = cklink.CKLink(dlldir=dir_dll, vid=self._cklink_vid, pid=self._cklink_pid, sn=serial, arch=2)
+            if not self.link:
+                self.link = cklink.CKLink(dlldir=dir_dll,
+                                          vid=self._cklink_vid,
+                                          pid=self._cklink_pid,
+                                          sn=serial,
+                                          arch=2,
+                                          cdi=0)
+                gol.obj_cklink = self.link
             #self.link.print_version()
             self.link.open()
             if self.link.connected():
-                self.link.reset(1)
+                self.link.reset(1)  # can not be set to 0
             return False
-        
+
     def if_close(self):
         if self.link:
             try:
-                self.link.halt()
+                #self.link.halt()
                 self.link.close()
             except Exception as e:
                 print(e)
             finally:
                 self._inited = False
 
+    def if_clear_buf(self):
+        pass
+
     def if_set_rx_timeout(self, val):
         self._rx_timeout = val * 1000
 
@@ -95,16 +105,18 @@ class BflbCKLinkPort(object):
 
     def halt_cpu(self):
         return self.link.halt()
-    
+
     def resume_cpu(self):
         return self.link.resume()
 
     def reset_cpu(self):
-        return self.link.reset(1) 
+        return self.link.reset(1)
 
     def set_pc_msp(self, pc, msp):
         self.halt_cpu()
-        if self._chiptype == "bl602" or self._chiptype == "bl702":
+        if self._chiptype == "bl602" \
+        or self._chiptype == "bl702"\
+        or self._chiptype == "bl702l":
             addr = int(self._cklink_run_addr, 16)
             self.link.write_cpu_reg(self._cklink_reg_pc, addr)
             #self.link.resume()
@@ -121,7 +133,7 @@ class BflbCKLinkPort(object):
         self.if_raw_write(self._cklink_data_addr, data_send)
         # write flag
         self.if_raw_write(self._cklink_shake_hand_addr, binascii.unhexlify("48524459"))
-        
+
     def if_read(self, data_len):
         start_time = (time.time() * 1000)
         while True:
@@ -139,7 +151,7 @@ class BflbCKLinkPort(object):
                 return 0, "waiting response time out".encode("utf-8")
             self.resume_cpu()
             time.sleep(0.001)
-        
+
         data = self.if_raw_read(self._cklink_data_addr, data_len)
         if len(data) != data_len:
             return 0, data
@@ -161,7 +173,7 @@ class BflbCKLinkPort(object):
                      reset_revert=True,
                      cutoff_time=0,
                      shake_hand_retry=2,
-                     iap_timeout=0,
+                     isp_timeout=0,
                      boot_load=False):
         self.if_write(bytearray(1))
         success, ack = self.if_read(2)
